@@ -8,17 +8,19 @@
 * The full license is in the file LICENSE, distributed with this software. *
 ****************************************************************************/
 
+#include "doctest/doctest.h"
+
 #include <algorithm>
 #include <chrono>
 #include <condition_variable>
 #include <cstdlib>
 #include <fstream>
+#include <iostream>
 #include <mutex>
 #include <thread>
 
 #include "xeus/xsystem.hpp"
 
-#include "gtest/gtest.h"
 #include "xeus_client.hpp"
 
 #include "pybind11/pybind11.h"
@@ -334,6 +336,21 @@ nl::json make_rich_inspect_variables_request(int seq, const std::string& var_nam
     return req;
 }
 
+nl::json make_copy_to_globals_request(int seq, const std::string& src_var_name, const std::string& dst_var_name, int src_frame_id)
+{
+    nl::json req = {
+        {"type", "request"},
+        {"seq", seq},
+        {"command", "copyToGlobals"},
+        {"arguments", {
+            {"srcVariableName", src_var_name},
+            {"dstVariableName", dst_var_name},
+            {"srcFrameId", src_frame_id}
+        }}
+    };
+    return req;
+}
+
 nl::json make_exception_breakpoint_request(int seq)
 {
     nl::json except_option = {
@@ -382,6 +399,7 @@ public:
     bool test_inspect_variables();
     bool test_rich_inspect_variables();
     bool test_variables();
+    bool test_copy_to_globals();
     void shutdown();
 
 private:
@@ -865,6 +883,88 @@ bool debugger_client::test_variables()
     return res;
 }
 
+bool debugger_client::test_copy_to_globals()
+{
+    std::string local_var_name = "var";
+    std::string global_var_name = "var_copy";
+    std::string code = "from IPython.core.display import HTML\ndef my_test():\n\t" + local_var_name + " = HTML(\"<p>test content</p>\")\n\tpass\nmy_test()";
+    int seq = 12;
+
+    // Init debugger and set breakpoint
+    attach();
+    m_client.send_on_control("debug_request", make_dump_cell_request(seq, code));
+    ++seq;
+    nl::json dump_res = m_client.receive_on_control();
+    std::string path = dump_res["content"]["body"]["sourcePath"].get<std::string>();
+    m_client.send_on_control("debug_request", make_breakpoint_request(seq, path, 4));
+    ++seq;
+    m_client.receive_on_control();
+
+    // Execute code
+    m_client.send_on_shell("execute_request", make_execute_request(code));
+
+    // Get breakpoint event
+    nl::json ev = m_client.wait_for_debug_event("stopped");
+    m_client.send_on_control("debug_request", make_stacktrace_request(seq, 1));
+    ++seq;
+    nl::json json1 = m_client.receive_on_control();
+    if(json1["content"]["body"]["stackFrames"].empty())
+    {
+        m_client.send_on_control("debug_request", make_stacktrace_request(seq, 1));
+        ++seq;
+        json1 = m_client.receive_on_control();
+    }
+
+    // Get local frame id
+    int frame_id = json1["content"]["body"]["stackFrames"][0]["id"].get<int>();
+
+    // Copy the variable
+    m_client.send_on_control("debug_request", make_copy_to_globals_request(seq, local_var_name, global_var_name, frame_id));
+    ++seq;
+
+    // Get the scopes
+    nl::json reply = m_client.receive_on_control();
+    m_client.send_on_control("debug_request", make_scopes_request(seq, frame_id));
+    ++seq;
+    nl::json json2 = m_client.receive_on_control();
+
+    // Get the local variable
+    int variable_ref_local = json2["content"]["body"]["scopes"][0]["variablesReference"].get<int>();
+    m_client.send_on_control("debug_request", make_variables_request(seq, variable_ref_local));
+    ++seq;
+    nl::json json3 = m_client.receive_on_control();
+    nl::json local_var = {};
+    for (auto &var: json3["content"]["body"]["variables"]){
+        if (var["evaluateName"] == local_var_name) {
+            local_var = var;
+        }
+    }
+    if (!local_var.contains("evaluateName")) {
+        std::cout << "Local variable \"" + local_var_name + "\" not found";
+        return false;
+    }
+
+    // Get the global variable (copy of the local variable)
+    nl::json global_scope = json2["content"]["body"]["scopes"].back();
+    int variable_ref_global = global_scope["variablesReference"].get<int>();
+    m_client.send_on_control("debug_request", make_variables_request(seq, variable_ref_global));
+    ++seq;
+    nl::json json4 = m_client.receive_on_control();
+    nl::json global_var = {};
+    for (auto &var: json4["content"]["body"]["variables"]){
+        if (var["evaluateName"] == global_var_name) {
+            global_var = var;
+        }
+    }
+    if (!global_var.contains("evaluateName")) {
+        std::cout << "Global variable \"" + global_var_name + "\" not found";
+        return false;
+    }
+
+    // Compare local and global variable
+    return global_var["value"] == local_var["value"] && global_var["type"] == local_var["type"];
+}
+
 void debugger_client::shutdown()
 {
     m_client.send_on_control("shutdown_request", make_shutdown_request());
@@ -940,6 +1040,62 @@ std::string debugger_client::make_external_invoker_code() const
     return "import external_code as ec\nec.my_test()\n";
 }
 
+/*********
+ * timer *
+ *********/
+
+class timer
+{
+public:
+
+    timer();
+    ~timer();
+
+    void notify_done();
+
+private:
+
+    void run_timer();
+
+    std::thread m_runner;
+    std::condition_variable m_cv;
+    std::mutex m_mcv;
+    bool m_done;
+};
+
+timer::timer()
+    : m_runner()
+    , m_cv()
+    , m_mcv()
+    , m_done(false)
+{
+    m_runner = std::move(std::thread(&timer::run_timer, this));
+}
+
+timer::~timer()
+{
+    m_runner.join();
+}
+
+void timer::notify_done()
+{
+    {
+        std::lock_guard<std::mutex> lk(m_mcv);
+        m_done = true;
+    }
+    m_cv.notify_one();
+}
+
+void timer::run_timer()
+{
+    std::unique_lock<std::mutex> lk(m_mcv);
+    if (!m_cv.wait_for(lk, std::chrono::seconds(20), [this]() { return m_done; }))
+    {
+        std::clog << "Unit test time out !!" << std::endl;
+        std::terminate();
+    }
+}
+
 /*********
  * tests *
  *********/
@@ -977,284 +1133,268 @@ void dump_connection_file()
 void start_kernel()
 {
     dump_connection_file();
-    std::thread kernel([]()
-    {
-        std::string cmd = "xpython -f " + KERNEL_JSON + "&";
-        int ret2 = std::system(cmd.c_str());
-    });
+    std::string cmd = "xpython -f " + KERNEL_JSON + "&";
+    int ret2 = std::system(cmd.c_str());
     std::this_thread::sleep_for(2s);
-    kernel.detach();
 }
 
-std::condition_variable cv;
-std::mutex mcv;
-bool done = false;
-
-void notify_done()
+TEST_SUITE("debugger")
 {
+    TEST_CASE("init")
     {
-        std::lock_guard<std::mutex> lk(mcv);
-        done = true;
+        start_kernel();
+        timer t;
+        zmq::context_t context;
+        {
+            debugger_client deb(context, KERNEL_JSON, "debugger_init.log");
+            bool res = deb.test_init();
+            deb.shutdown();
+            std::this_thread::sleep_for(2s);
+            CHECK(res);
+            t.notify_done();
+        }
     }
-    cv.notify_one();
-}
 
-void run_timer()
-{
-    std::unique_lock<std::mutex> lk(mcv);
-    if (!cv.wait_for(lk, std::chrono::seconds(20), []() { return done; }))
+    TEST_CASE("disconnect")
     {
-        std::clog << "Unit test time out !!" << std::endl;
-        std::terminate();
-    }
-}
-
-void start_timer()
-{
-    done = false;
-    std::thread t(run_timer);
-    t.detach();
-}
-
-TEST(debugger, init)
-{
-    start_kernel();
-    start_timer();
-    zmq::context_t context;
-    {
-        debugger_client deb(context, KERNEL_JSON, "debugger_init.log");
-        bool res = deb.test_init();
-        deb.shutdown();
-        std::this_thread::sleep_for(2s);
-        EXPECT_TRUE(res);
-        notify_done();
-    }
-}
-
-TEST(debugger, disconnect)
-{
-    start_kernel();
-    start_timer();
-    zmq::context_t context;
-    {
-        debugger_client deb(context, KERNEL_JSON, "debugger_disconnect.log");
-        bool res = deb.test_disconnect();
-        deb.shutdown();
-        std::this_thread::sleep_for(2s);
-        EXPECT_TRUE(res);
-        notify_done();
-    }
-}
-
-TEST(debugger, attach)
-{
-    start_kernel();
-    start_timer();
-    zmq::context_t context;
-    {
-        debugger_client deb(context, KERNEL_JSON, "debugger_attach.log");
-        bool res = deb.test_attach();
-        deb.shutdown();
-        std::this_thread::sleep_for(2s);
-        EXPECT_TRUE(res);
-        notify_done();
-    }
-}
-
-TEST(debugger, multisession)
-{
-    start_kernel();
-    start_timer();
-    zmq::context_t context;
-    {
-        debugger_client deb(context, KERNEL_JSON, "debugger_multi_session.log");
-        bool res1 = deb.test_disconnect();
-        std::this_thread::sleep_for(2s);
-        bool res2 = deb.test_disconnect();
-        deb.shutdown();
-        std::this_thread::sleep_for(2s);
-        EXPECT_TRUE(res1);
-        EXPECT_TRUE(res2);
-        notify_done();
-    }
-}
-
-TEST(debugger, set_external_breakpoints)
-{
-    start_kernel();
-    start_timer();
-    zmq::context_t context;
-    {
-        debugger_client deb(context, KERNEL_JSON, "debugger_set_external_breakpoints.log");
-        bool res = deb.test_external_set_breakpoints();
-        deb.shutdown();
-        std::this_thread::sleep_for(2s);
-        EXPECT_TRUE(res);
-        notify_done();
-    }
-}
-
-TEST(debugger, external_next_continue)
-{
-    start_kernel();
-    start_timer();
-    zmq::context_t context;
-    {
-        debugger_client deb(context, KERNEL_JSON, "debugger_external_next_continue.log");
-        bool res = deb.test_external_next_continue();
-        deb.shutdown();
-        std::this_thread::sleep_for(2s);
-        EXPECT_TRUE(res);
-        notify_done();
-    }
-}
-
-TEST(debugger, set_breakpoints)
-{
-    start_kernel();
-    start_timer();
-    zmq::context_t context;
-    {
-        debugger_client deb(context, KERNEL_JSON, "debugger_set_breakpoints.log");
-        bool res = deb.test_set_breakpoints();
-        deb.shutdown();
-        std::this_thread::sleep_for(2s);
-        EXPECT_TRUE(res);
-        notify_done();
-    }
-}
-
-TEST(debugger, set_exception_breakpoints)
-{
-    start_kernel();
-    start_timer();
-    zmq::context_t context;
-    {
-        debugger_client deb(context, KERNEL_JSON, "debugger_set_exception_breakpoints.log");
-        bool res = deb.test_set_exception_breakpoints();
-        deb.shutdown();
-        std::this_thread::sleep_for(5s);
-        EXPECT_TRUE(res);
-        notify_done();
-    }
-}
-
-TEST(debugger, source)
-{
-    start_kernel();
-    start_timer();
-    zmq::context_t context;
-    {
-        debugger_client deb(context, KERNEL_JSON, "debugger_source.log");
-        bool res = deb.test_source();
-        deb.shutdown();
-        std::this_thread::sleep_for(2s);
-        EXPECT_TRUE(res);
-        notify_done();
-    }
-}
-
-TEST(debugger, next_continue)
-{
-    start_kernel();
-    start_timer();
-    zmq::context_t context;
-    {
-        debugger_client deb(context, KERNEL_JSON, "debugger_next_continue.log");
-        bool res = deb.test_next_continue();
-        deb.shutdown();
-        std::this_thread::sleep_for(2s);
-        EXPECT_TRUE(res);
-        notify_done();
-    }
-}
-
-TEST(debugger, stepin)
-{
-    start_kernel();
-    start_timer();
-    zmq::context_t context;
-    {
-        debugger_client deb(context, KERNEL_JSON, "debugger_stepin.log");
-        bool res = deb.test_step_in();
-        deb.shutdown();
-        std::this_thread::sleep_for(2s);
-        EXPECT_TRUE(res);
-        notify_done();
-    }
-}
-
-TEST(debugger, stack_trace)
-{
-    start_kernel();
-    start_timer();
-    zmq::context_t context;
-    {
-        debugger_client deb(context, KERNEL_JSON, "debugger_stack_trace.log");
-        bool res = deb.test_stack_trace();
-        deb.shutdown();
-        std::this_thread::sleep_for(2s);
-        EXPECT_TRUE(res);
-        notify_done();
-    }
-}
-
-TEST(debugger, debug_info)
-{
-    start_kernel();
-    start_timer();
-    zmq::context_t context;
-    {
-        debugger_client deb(context, KERNEL_JSON, "debugger_debug_info.log");
-        bool res = deb.test_debug_info();
-        deb.shutdown();
-        std::this_thread::sleep_for(2s);
-        EXPECT_TRUE(res);
-        notify_done();
-    }
-}
-
-TEST(debugger, inspect_variables)
-{
-    start_kernel();
-    start_timer();
-    zmq::context_t context;
-    {
-        debugger_client deb(context, KERNEL_JSON, "debugger_inspect_variables.log");
-        bool res = deb.test_inspect_variables();
-        deb.shutdown();
-        std::this_thread::sleep_for(2s);
-        EXPECT_TRUE(res);
-        notify_done();
-    }
-}
-
-TEST(debugger, rich_inspect_variables)
-{
-    start_kernel();
-    start_timer();
-    zmq::context_t context;
-    {
-        debugger_client deb(context, KERNEL_JSON, "debugger_rich_inspect_variables.log");
-        bool res = deb.test_rich_inspect_variables();
-        deb.shutdown();
-        std::this_thread::sleep_for(2s);
-        EXPECT_TRUE(res);
-        notify_done();
-    }
-}
-
-TEST(debugger, variables)
-{
-    start_kernel();
-    start_timer();
-    zmq::context_t context;
-    {
-        debugger_client deb(context, KERNEL_JSON, "debugger_variables.log");
-        bool res = deb.test_variables();
-        deb.shutdown();
-        std::this_thread::sleep_for(2s);
-        EXPECT_TRUE(res);
-        notify_done();
+        start_kernel();
+        timer t;
+        zmq::context_t context;
+        {
+            debugger_client deb(context, KERNEL_JSON, "debugger_disconnect.log");
+            bool res = deb.test_disconnect();
+            deb.shutdown();
+            std::this_thread::sleep_for(2s);
+            CHECK(res);
+            t.notify_done();
+        }
+    }
+
+    TEST_CASE("attach")
+    {
+        start_kernel();
+        timer t;
+        zmq::context_t context;
+        {
+            debugger_client deb(context, KERNEL_JSON, "debugger_attach.log");
+            bool res = deb.test_attach();
+            deb.shutdown();
+            std::this_thread::sleep_for(2s);
+            CHECK(res);
+            t.notify_done();
+        }
+    }
+
+    TEST_CASE("multisession")
+    {
+        start_kernel();
+        timer t;
+        zmq::context_t context;
+        {
+            debugger_client deb(context, KERNEL_JSON, "debugger_multi_session.log");
+            bool res1 = deb.test_disconnect();
+            std::this_thread::sleep_for(2s);
+            bool res2 = deb.test_disconnect();
+            deb.shutdown();
+            std::this_thread::sleep_for(2s);
+            CHECK(res1);
+            CHECK(res2);
+            t.notify_done();
+        }
+    }
+
+    TEST_CASE("set_external_breakpoints")
+    {
+        start_kernel();
+        timer t;
+        zmq::context_t context;
+        {
+            debugger_client deb(context, KERNEL_JSON, "debugger_set_external_breakpoints.log");
+            bool res = deb.test_external_set_breakpoints();
+            deb.shutdown();
+            std::this_thread::sleep_for(2s);
+            CHECK(res);
+            t.notify_done();
+        }
+    }
+
+    TEST_CASE("external_next_continue")
+    {
+        start_kernel();
+        timer t;
+        zmq::context_t context;
+        {
+            debugger_client deb(context, KERNEL_JSON, "debugger_external_next_continue.log");
+            bool res = deb.test_external_next_continue();
+            deb.shutdown();
+            std::this_thread::sleep_for(2s);
+            CHECK(res);
+            t.notify_done();
+        }
+    }
+
+    TEST_CASE("set_breakpoints")
+    {
+        start_kernel();
+        timer t;
+        zmq::context_t context;
+        {
+            debugger_client deb(context, KERNEL_JSON, "debugger_set_breakpoints.log");
+            bool res = deb.test_set_breakpoints();
+            deb.shutdown();
+            std::this_thread::sleep_for(2s);
+            CHECK(res);
+            t.notify_done();
+        }
+    }
+
+    TEST_CASE("set_exception_breakpoints")
+    {
+        start_kernel();
+        timer t;
+        zmq::context_t context;
+        {
+            debugger_client deb(context, KERNEL_JSON, "debugger_set_exception_breakpoints.log");
+            bool res = deb.test_set_exception_breakpoints();
+            deb.shutdown();
+            std::this_thread::sleep_for(5s);
+            CHECK(res);
+            t.notify_done();
+        }
+    }
+
+    TEST_CASE("source")
+    {
+        start_kernel();
+        timer t;
+        zmq::context_t context;
+        {
+            debugger_client deb(context, KERNEL_JSON, "debugger_source.log");
+            bool res = deb.test_source();
+            deb.shutdown();
+            std::this_thread::sleep_for(2s);
+            CHECK(res);
+            t.notify_done();
+        }
+    }
+
+    TEST_CASE("next_continue")
+    {
+        start_kernel();
+        timer t;
+        zmq::context_t context;
+        {
+            debugger_client deb(context, KERNEL_JSON, "debugger_next_continue.log");
+            bool res = deb.test_next_continue();
+            deb.shutdown();
+            std::this_thread::sleep_for(2s);
+            CHECK(res);
+            t.notify_done();
+        }
+    }
+
+    TEST_CASE("stepin")
+    {
+        start_kernel();
+        timer t;
+        zmq::context_t context;
+        {
+            debugger_client deb(context, KERNEL_JSON, "debugger_stepin.log");
+            bool res = deb.test_step_in();
+            deb.shutdown();
+            std::this_thread::sleep_for(2s);
+            CHECK(res);
+            t.notify_done();
+        }
+    }
+
+    TEST_CASE("stack_trace")
+    {
+        start_kernel();
+        timer t;
+        zmq::context_t context;
+        {
+            debugger_client deb(context, KERNEL_JSON, "debugger_stack_trace.log");
+            bool res = deb.test_stack_trace();
+            deb.shutdown();
+            std::this_thread::sleep_for(2s);
+            CHECK(res);
+            t.notify_done();
+        }
+    }
+
+    TEST_CASE("debug_info")
+    {
+        start_kernel();
+        timer t;
+        zmq::context_t context;
+        {
+            debugger_client deb(context, KERNEL_JSON, "debugger_debug_info.log");
+            bool res = deb.test_debug_info();
+            deb.shutdown();
+            std::this_thread::sleep_for(2s);
+            CHECK(res);
+            t.notify_done();
+        }
+    }
+
+    TEST_CASE("inspect_variables")
+    {
+        start_kernel();
+        timer t;
+        zmq::context_t context;
+        {
+            debugger_client deb(context, KERNEL_JSON, "debugger_inspect_variables.log");
+            bool res = deb.test_inspect_variables();
+            deb.shutdown();
+            std::this_thread::sleep_for(2s);
+            CHECK(res);
+            t.notify_done();
+        }
+    }
+
+    TEST_CASE("rich_inspect_variables")
+    {
+        start_kernel();
+        timer t;
+        zmq::context_t context;
+        {
+            debugger_client deb(context, KERNEL_JSON, "debugger_rich_inspect_variables.log");
+            bool res = deb.test_rich_inspect_variables();
+            deb.shutdown();
+            std::this_thread::sleep_for(2s);
+            CHECK(res);
+            t.notify_done();
+        }
+    }
+
+    TEST_CASE("variables")
+    {
+        start_kernel();
+        timer t;
+        zmq::context_t context;
+        {
+            debugger_client deb(context, KERNEL_JSON, "debugger_variables.log");
+            bool res = deb.test_variables();
+            deb.shutdown();
+            std::this_thread::sleep_for(2s);
+            CHECK(res);
+            t.notify_done();
+        }
+    }
+
+    TEST_CASE("copy_to_globals")
+    {
+        start_kernel();
+        timer t;
+        zmq::context_t context;
+        {
+            debugger_client deb(context, KERNEL_JSON, "debugger_copy_to_globals.log");
+            bool res = deb.test_copy_to_globals();
+            deb.shutdown();
+            std::this_thread::sleep_for(2s);
+            CHECK(res);
+            t.notify_done();
+        }
     }
 }
